/* <stmt> := var <ident> = <expr>
	| <ident> = <expr>
	| for <ident> = <expr> to <expr> do <stmt> end
	| read_int <ident>
	| print <expr>
	| <stmt> ; <stmt>
  */
using System;
using System.Text;

public abstract class Stmt
{
    public abstract void ToString(StringBuilder sb, int inden);
    public static void WriteIdent(StringBuilder sb, int inden)
    {
        for (int i = inden - 1; i >= 0; i--)
        {
           sb.Append(" ");
           if (i % 2 == 0)
               sb.Append("|");
        }
    }
}
/* <expr> := <string>
 *  | <int>
 *  | <arith_expr>
 *  | <ident>
 */

public abstract class Expr {
    public abstract void ToString(StringBuilder sb, int inden);
    public static void WriteIdent(StringBuilder sb, int inden) {
        for (int i = inden - 1; i >= 0; i--) {
            sb.Append(" ");
            if (i % 2 == 0)
                sb.Append("|");
        }
    }
}

// var <ident> = <expr>
public class DeclareVar : Stmt
{
    public Expr Expr;
    public string Ident;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.Append("[DeclareVar]\r\n");
        WriteIdent(sb, inden+2);
        sb.AppendFormat("Ident:{0}\r\n", Ident);
        WriteIdent(sb, inden + 2);
        sb.Append("Expr:\r\n");
        Expr.ToString(sb, inden + 4);
    }
}

// print <expr>
public class Print : Stmt
{
    public Expr Expr;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.Append("[Print]\r\n");
        WriteIdent(sb, inden + 2);
        sb.Append("Expr:\r\n");
        Expr.ToString(sb, inden + 4);
    }
}

// <ident> = <expr>
public class Assign : Stmt
{
    public Expr Expr;
    public string Ident;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.Append("[DeclareVar]\r\n");
        WriteIdent(sb, inden + 2);
        sb.AppendFormat("Ident:{0}\r\n", inden);
        WriteIdent(sb, inden + 2);
        sb.Append("Expr:\r\n");
        Expr.ToString(sb, inden + 4);
    }
}

// for <ident> = <expr> to <expr> do <stmt> end
public class ForLoop : Stmt
{
    public Stmt Body;
    public Expr From;
    public string Ident;
    public Expr To;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.Append("[ForLoop]\r\n");
        WriteIdent(sb, inden + 2);
        sb.AppendFormat("Ident:{0}\r\n", inden);
        WriteIdent(sb, inden + 2);
        sb.Append("From:\r\n");
        From.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("To:\r\n");
        To.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("Body:\r\n");
        Body.ToString(sb, inden + 4);
    }
}

// read_int <ident>
public class ReadInt : Stmt
{
    public string Ident;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.Append("[ReadInt]\r\n");
        WriteIdent(sb, inden + 2);
        sb.AppendFormat("Ident:{0}\r\n", inden);
    }
}

// <stmt> ; <stmt>
public class Sequence : Stmt
{
    public Stmt First;
    public Stmt Second;
    public override void ToString(StringBuilder sb, int inden)
    {

        WriteIdent(sb, inden);
        sb.Append("[Sequence]\r\n");
        WriteIdent(sb, inden + 2);
        sb.Append("First:\r\n");
        First.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("Second:\r\n");
        Second.ToString(sb, inden + 4);
    }
}


// <string> := " <string_elem>* "
public class StringLiteral : Expr
{
    public string Value;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<StringLiteral>:\"{0}\"\r\n", Value);
    }
}

// <int> := <digit>+
public class IntLiteral : Expr
{
    public int Value;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<IntLiteral>:{0}\r\n", Value);
    }
}

// <ident> := <char> <ident_rest>*
// <ident_rest> := <char> | <digit>
public class Variable : Expr
{
    public string Ident;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<Variable>:{0}\r\n", Ident);
    }
}

// <bin_expr> := <expr> <bin_op> <expr>
public class BinExpr : Expr
{
    public Expr Left;
    public BinOp Op;
    public Expr Right;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<BinExpr>:{0}\r\n", Op);
        WriteIdent(sb, inden+2);
        sb.Append("left:\r\n");
        Left.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("Right:\r\n");
        Right.ToString(sb, inden + 4);

    }
}

// <bin_op> := + | - | * | /
public enum BinOp
{
    Add,
    Sub,
    Mul,
    Div,
    Eq,
}

//match <expr> <expr>
public class Match : Expr
{
    public Expr Input;
    public Expr Pattern;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<Match>:\r\n");
        WriteIdent(sb, inden+2);
        sb.Append("Input:\r\n");
        Input.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("Pattern:\r\n");
        Pattern.ToString(sb, inden + 4);
    }
}

//sb()
public class Builder : Expr
{
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.Append("[Builder]\r\n");
    }
}

//foreach <ident> in <expr> do <stmt> end
public class Foreach : Stmt
{
    public Stmt Body;
    public string Ident;
    public Expr IEnumerable;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<Foreach>:{0}\r\n", Ident);
        WriteIdent(sb, inden+2);
        sb.Append("IEnumerable:\r\n");
        IEnumerable.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("Body:\r\n");
        Body.ToString(sb, inden + 4);
    }
}

//if <expr> then <stmt> end
public class If : Stmt {
    public Stmt Body;
    public Expr Condition;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<If>:\r\n");
        WriteIdent(sb, inden+2);
        sb.Append("Condition:\r\n");
        Condition.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("Body:\r\n");
        Body.ToString(sb, inden + 4);
    }
}

//len <expr>
public class StrLen : Expr
{
    public Expr Input;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("<StrLen>:\r\n");
        WriteIdent(sb, inden+2);
        sb.Append("Input:\r\n");
        Input.ToString(sb, inden + 4);
    }
}
//append <expr> <expr>
public class Append : Stmt
{
    public Expr Buider;
    public Expr ToAppend;
    public override void ToString(StringBuilder sb, int inden)
    {
        WriteIdent(sb, inden);
        sb.AppendFormat("[Append]:\r\n");
        WriteIdent(sb, inden + 2);
        sb.Append("Buider:\r\n");
        Buider.ToString(sb, inden + 4);
        WriteIdent(sb, inden + 2);
        sb.Append("ToAppend:\r\n");
        ToAppend.ToString(sb, inden + 4);
    }
}